module Ephemeris::Zodiac
  module Aspect
    def self.orb_range_defined_by_target( planet_1, planet_2 )
      coverage = (planet_1.class::ORB + planet_2.class::ORB).to_f / 2
      point = planet_2.position

      if point < coverage or point+coverage > 360
        return self.orb_range_by_target_at_aries_pisces( planet_1, planet_2, coverage)
      else
        return Ephemeris::Zodiac::Orb.new( (point - coverage)..(point + coverage), planet_1 )
      end
    end

    def self.orb_range_by_target_at_aries_pisces( planet_1, planet_2, coverage)
      # Try it the other way around
      point = planet_1.position

      if point < coverage or point+coverage > 360
        # Ok, now for the hard part
        if point < coverage
          # Aries with a lower Pisces
          pisces_range = [340, 341, 342, 343, 344, 345, 346, 347, 348, 349, 350, 351, 352, 353, 354, 355, 356, 357, 358, 359]
          difference = (point - coverage).round
          pisces_range = pisces_range[ difference ]...360
          return Ephemeris::Zodiac::Orb.new( [ pisces_range, 0..point+coverage], planet_2 )
        else
          # Pisces side upper Aries
          difference = point + coverage - 360
          return Ephemeris::Zodiac::Orb.new( [point-coverage...360, 0...difference], planet_2 )
        end
      else
        return Ephemeris::Zodiac::Orb.new( (point - coverage)..(point + coverage), planet_2 )
      end
    end

    module Conjunction
      def self.conjoined?( planet_1, planet_2 )
        orb = Ephemeris::Zodiac::Aspect.orb_range_defined_by_target( planet_1, planet_2 )
        orb.cover?( [0] )
      end
    end

    module Opposition
      def self.opposed?( planet_1, planet_2 )
        orb = Ephemeris::Zodiac::Aspect.orb_range_defined_by_target( planet_1, planet_2 )
        calc = self.build_appropriate_motion_from_position( orb.target.position )

        orb.cover?( calc )
      end

      def self.build_appropriate_motion_from_position( position )
        return [ -180 ] if ( position >= 180 )
        return [ 180 ] if ( position < 180 )
      end
    end

    module Square
      def self.square?(planet_1, planet_2)
        orb = Ephemeris::Zodiac::Aspect.orb_range_defined_by_target( planet_1, planet_2 )
        calc = self.build_appropriate_motion_from_position( orb.target.position )

        orb.cover?( calc ) #already an array
      end

      def self.build_appropriate_motion_from_position( position )
        if (0...90).cover? position
          return [ 90, 270 ]
        elsif (90...270).cover? position
          return [ -90, 90 ]
        elsif (270...360).cover? position
          return [ -90, -270 ]
        else
          raise StandardError.new("There are only 360 degress in a circle, buddy.")
        end
      end
    end

    module Trine
      def self.trine?( planet_1, planet_2 )
        orb = Ephemeris::Zodiac::Aspect.orb_range_defined_by_target( planet_1, planet_2 )
        calc = self.build_appropriate_motion_from_position( orb.target.position )

        orb.cover?( calc ) # already an array
      end

      def self.build_appropriate_motion_from_position( position )
        if (0...120).cover? position
          return [ 120, 240 ]
        elsif (120...240).cover? position
          return [ 120, -120 ]
        elsif (240...360).cover? position
          return [ -120, -240 ]
        else
          raise StandardError.new("There are only 360 degress in a circle, buddy.")
        end
      end
    end

    module Sextile
      def self.sextile?( planet_1, planet_2 )
        orb = Ephemeris::Zodiac::Aspect.orb_range_defined_by_target( planet_1, planet_2 )
        calc = self.build_appropriate_motion_from_position( orb.target.position )

        orb.cover?( calc ) # already an array
      end

      def self.build_appropriate_motion_from_position( position )
        if (0...60).cover? position
          return [ 60, 300 ]
        elsif (60...300).cover? position
          return [ -60, 60 ]
        elsif (240...360).cover? position
          return [ -60, -300 ]
        else
          raise StandardError.new("There are only 360 degress in a circle, buddy.")
        end
      end
    end
  end
end
